# test.utils_tests.timez_tests
# Testing for the timez time helpers library.
#
# Author:   Benjamin Bengfort <benjamin@bengfort.com>
# Created:  Sun Feb 21 15:33:18 2016 -0500
#
# Copyright (C) 2016 Bengfort.com
# For license information, see LICENSE.txt
#
# ID: timez_tests.py [] benjamin@bengfort.com $

"""
Testing for the timez time helpers library.
"""

##########################################################################
## Imports
##########################################################################

import unittest

from datetime import datetime
from dateutil.tz import tzutc
from baleen.utils.timez import *

##########################################################################
## Time Helper Functions Tests
##########################################################################

class TimezTests(unittest.TestCase):

    def test_non_naive_datetimes(self):
        """
        Assert that localnow and utcnow return non-naive datetimes
        """
        self.assertIsNotNone(localnow().tzinfo)
        self.assertIsNotNone(utcnow().tzinfo)

    def test_strptimez(self):
        """
        Test the parsing of timezone aware date strings
        """
        dtfmt = "%Y-%m-%dT%H:%M:%S%z"

        cases = (
            ('2012-12-27T12:53:12-0500', datetime(2012, 12, 27, 17, 53, 12, tzinfo=tzutc())),
            ('2012-12-27T12:53:12+0800', datetime(2012, 12, 27, 4, 53, 12, tzinfo=tzutc())),
        )

        for dtstr, dt in cases:
            self.assertEqual(dt, strptimez(dtstr, dtfmt))

        # Non-timezone case
        self.assertEqual(
            strptimez('2012-12-27T12:53:12', "%Y-%m-%dT%H:%M:%S"),
            datetime(2012, 12, 27, 12, 53, 12)
        )



# tests
# Testing for the baleen module
#
# Author:   Benjamin Bengfort <benjamin@bengfort.com>
# Created:  Fri Sep 19 10:58:15 2014 -0400
#
# Copyright (C) 2014 Bengfort.com
# For license information, see LICENSE.txt
#
# ID: __init__.py [] benjamin@bengfort.com $

"""
Testing for the baleen module
"""

##########################################################################
## Imports
##########################################################################

import unittest

##########################################################################
## Module Constants
##########################################################################

TEST_VERSION = "0.2" ## Also the expected version onf the package

##########################################################################
## Test Cases
##########################################################################

class InitializationTest(unittest.TestCase):

    def test_initialization(self):
        """
        Tests a simple world fact by asserting that 10*10 is 100
        """
        self.assertEqual(10*10, 100)

    def test_import(self):
        """
        Can import baleen
        """
        try:
            import baleen
        except ImportError:
            self.fail("Unable to import the baleen module!")

    def test_version(self):
        """
        Assert that the version is sane
        """
        import baleen
        self.assertEqual(TEST_VERSION, baleen.__version__)



# baleen.models
# Object Document Models for use with Mongo and mongoengine
#
# Author:   Benjamin Bengfort <benjamin@bengfort.com>
# Created:  Fri Sep 19 11:30:53 2014 -0400
#
# Copyright (C) 2014 Bengfort.com
# For license information, see LICENSE.txt
#
# ID: models.py [] benjamin@bengfort.com $

"""
Object Document Models for use with Mongo and mongoengine
"""

##########################################################################
## Imports
##########################################################################

import hashlib
import mongoengine as me

from datetime import datetime
from baleen.config import settings

##########################################################################
## Module Constants
##########################################################################

FEEDTYPES = (
    'atom',
    'atom01',
    'atom02',
    'atom03',
    'atom10',
    'cdf',
    'rss',
    'rss090',
    'rss091n',
    'rss092',
    'rss093',
    'rss094',
    'rss10',
    'rss20',
)

##########################################################################
## Helper Functions
##########################################################################

def connect(**kwargs):
    """
    Wrapper for mongoengine connect - connects with configuration details.
    """
    name = kwargs.get('name', settings.database.name)
    host = kwargs.get('host', settings.database.host)
    port = kwargs.get('port', settings.database.port)

    return me.connect(name, host=host, port=port)

##########################################################################
## Models
##########################################################################

class Feed(me.DynamicDocument):

    version   = me.StringField(choices=FEEDTYPES)
    etag      = me.StringField()
    modified  = me.StringField()
    title     = me.StringField(max_length=256)
    link      = me.URLField(required=True, unique=True)
    urls      = me.DictField()
    category  = me.StringField(required=True)
    active    = me.BooleanField(default=True)
    fetched   = me.DateTimeField(default=None)
    created   = me.DateTimeField(default=datetime.now, required=True)
    updated   = me.DateTimeField(default=datetime.now, required=True)

    @classmethod
    def pre_save(cls, sender, document, **kwargs):
        document.updated = datetime.now()

    meta      = {
        'collection': 'feeds',
    }

    @property
    def xmlurl(self):
        return self.link

    @property
    def htmlurl(self):
        return self.urls.get('htmlurl')

    def __unicode__(self):
        return self.title

class Post(me.DynamicDocument):

    feed      = me.ReferenceField(Feed)
    title     = me.StringField( max_length=512 )
    url       = me.URLField( required=True, unique=True )
    pubdate   = me.DateTimeField()
    content   = me.StringField( required=True )
    tags      = me.ListField(me.StringField(max_length=256))
    signature = me.StringField( required=True, max_length=64, min_length=64, unique=True )
    created   = me.DateTimeField(default=datetime.now, required=True)
    updated   = me.DateTimeField(default=datetime.now, required=True)

    @classmethod
    def pre_save(cls, sender, document, **kwargs):
        document.updated   = datetime.now()
        document.signature = document.hash()

    meta      = {
        'collection': 'posts',
    }

    def hash(self):
        """
        Returns the SHA256 hash of the content.
        """
        sha = hashlib.sha256()
        sha.update(self.content.encode('UTF-8'))
        return sha.hexdigest()

    def htmlize(self):
        """
        Returns an HTML string of the content of the Post
        """

        template = (
            u"<!doctype html>\n"
            u"<html>\n"
            u"<head>\n\t<meta charset=\"utf-8\"\n>"
            u"\t<title>%s</title>\n</head>\n"
            u"<body>\n\n%s\n\n</body>\n</html>\n"
        )

        return template % (self.title, self.content)

    def __unicode__(self):
        return self.title

##########################################################################
## Signals
##########################################################################

me.signals.pre_save.connect(Feed.pre_save, sender=Feed)
me.signals.pre_save.connect(Post.pre_save, sender=Post)



# baleen.utils.mongolog
# Handlers and formatters for logging to Mongo
#
# Author:   Benjamin Bengfort <benjamin@bengfort.com>
# Created:  Tue Sep 23 09:11:52 2014 -0400
#
# Copyright (C) 2014 Bengfort.com
# For license information, see LICENSE.txt
#
# ID: mongolog.py [] benjamin@bengfort.com $

"""
Handlers and formatters for logging to Mongo
"""

##########################################################################
## Imports
##########################################################################

import getpass
import logging
import logging.config
from baleen.utils.timez import *
from baleen.config import settings

from datetime import datetime
from socket import gethostname
from pymongo import MongoClient
from pymongo.errors import OperationFailure, PyMongoError

##########################################################################
## Mongo Formatter/Handler
##########################################################################

class MongoFormatter(logging.Formatter):

    def __init__(self, fmt='%(name)s %(levelname)s [%(asctime)s] -- %(message)s', datefmt=COMMON_DATETIME):
        super(MongoFormatter, self).__init__(fmt, datefmt)

    def format(self, record):
        """
        Formats LogRecord into a Python dictionary
        """

        ## Get the dictionary ready for Mongo
        data    = record.__dict__.copy()

        ## Get the log message as intended via super
        message   = super(MongoFormatter, self).format(record)
        timestamp = datetime.fromtimestamp(data.pop('created'))
        location  = {
            'module': data.pop('module'),
            'file': data.pop('pathname'),
            'filename': data.pop('filename'),
            'lineno': data.pop('lineno'),
            'method': data.pop('funcName')
        }
        error     = {
            'info': data.pop('exc_info'),
            'text': data.pop('exc_text'),
        }
        process   = {
            'process': data.pop('process'),
            'processName': data.pop('processName'),
            'thread': data.pop('thread'),
            'threadName': data.pop('threadName'),
        }
        logger    = data.pop('name')
        level     = {
            'number': data.pop('levelno'),
            'name': data.pop('levelname'),
        }
        info      = tuple(unicode(arg) for arg in data.pop('args'))

        for key in ('relativeCreated', 'msecs', 'msg'):
            del data[key]

        data.update({
            'logger': logger,
            # 'process': process,
            'message': message,
            'timestamp': timestamp,
            'level': level,
            # 'location': location,
            'error': error,
            'user': getpass.getuser(),
            'host': gethostname(),
            # 'info': info,
        })

        return data

class MongoHandler(logging.Handler):

    def __init__(self, level=logging.NOTSET, **kwargs):
        super(MongoHandler, self).__init__(level)
        self.host            = kwargs.get('host', settings.database.host)
        self.port            = kwargs.get('port', settings.database.port)
        self.database_name   = kwargs.get('database', settings.database.name)
        self.collection_name = kwargs.get('collection', 'logs')
        self.fail_silently   = kwargs.get('fail_silently', False)
        self.formatter       = kwargs.get('formatter', MongoFormatter())

        self.connection      = None
        self.database        = None
        self.collection      = None
        self.connect()

    def connect(self):
        """
        Connect to the Mongo database.
        """
        try:
            self.connection = MongoClient(host=self.host, port=self.port)
        except PyMongoError:
            if self.fail_silently:
                return
            else:
                raise

        self.database   = self.connection[self.database_name]
        self.collection = self.database[self.collection_name]

    def close(self):
        """
        Close the connection to the Mongo database.
        """
        if self.connection is not None:
            self.connection = None

    def emit(self, record):
        """
        Insert log record into Mongo database
        """
        if self.collection is not None:
            try:
                self.collection.insert(self.format(record))
            except Exception:
                if not self.fail_silently:
                    self.handleError(record)

if __name__ == '__main__':
    logger = logging.getLogger('demo')
    logger.setLevel(logging.INFO)
    logger.addHandler(MongoHandler())
    logger.info("This is a test of the logging system")



# baleen.utils.logger
# Logging utility for Baleen
#
# Author:   Benjamin Bengfort <benjamin@bengfort.com>
# Created:  Mon Sep 22 15:47:34 2014 -0400
#
# Copyright (C) 2014 Bengfort.com
# For license information, see LICENSE.txt
#
# ID: logger.py [] benjamin@bengfort.com $

"""
Logging utility for Baleen
"""

##########################################################################
## Imports
##########################################################################

import logging
import logging.config
from baleen.utils.timez import *
from baleen.config import settings

##########################################################################
## Logging configuration
##########################################################################

DEFAULT_LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'simple': {
            'format': '%(name)s %(levelname)s [%(asctime)s] -- %(message)s',
            'datefmt': COMMON_DATETIME,
        }
    },
    'handlers': {
        'null': {
            'level': 'DEBUG',
            'class': 'logging.NullHandler',
        },
        'console': {
            'level': 'WARNING',
            'class': 'logging.StreamHandler',
            'formatter': 'simple',
        },
        'logfile': {
            'level': 'INFO',
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': settings.logfile,
            'maxBytes': '536870912', # 512 MB
            'formatter': 'simple',
        },
        'mongolog': {
            'level': 'INFO',
            'class': 'baleen.utils.mongolog.MongoHandler',
        }
    },
    'loggers': {
        'baleen': {
            'level': settings.loglevel,
            'handlers': ['logfile'],
            'propagagte': True,
        },
        'baleen.ingest': {
            'level': 'INFO',
            'handlers': ['logfile', 'mongolog'],
            'propagate': False,
        }
    },
}

# SET LOGGING CONFIG HERE!!
logging.config.dictConfigClass(DEFAULT_LOGGING).configure()

##########################################################################
## Logger Class
##########################################################################

class Logger(object):
    """
    Helper class for performing logging with the correct configuration,
    and also ensures that the log configuration is imported. Subclasses
    must specify their logger as a class variable so that all instances
    have access to the same log.
    """

    logger = logging.getLogger('baleen')

    def __init__(self, **kwargs):
        self.logger = kwargs.pop('logger', self.logger)
        if not self.logger or not hasattr(self.logger, 'log'):
            raise NotImplementedError("Subclasses must specify a logger.")

        self.extras = kwargs

    def log(self, level, message, *args, **kwargs):
        """
        Subclasses may override this to ensure proper logging with extra
        options gets correctly specified. See the `ApacheLogger` for more
        information on how to specify Common Log format.
        """
        extra = kwargs.pop('extra', {})
        extra.update(self.extras)
        kwargs['extra'] = extra
        self.logger.log(level, message, *args, **kwargs)

    def debug(self, message, *args, **kwargs):
        return self.log(logging.DEBUG, message, *args, **kwargs)

    def info(self, message, *args, **kwargs):
        return self.log(logging.INFO, message, *args, **kwargs)

    def warning(self, message, *args, **kwargs):
        return self.log(logging.WARNING, message, *args, **kwargs)

    def error(self, message, *args, **kwargs):
        return self.log(logging.ERROR, message, *args, **kwargs)

    def critical(self, message, *args, **kwargs):
        return self.log(logging.CRITICAL, message, *args, **kwargs)


class IngestLogger(Logger):
    """
    Performs logging for the baleen process with the log options above.
    """

    logger = logging.getLogger('baleen.ingest')



# baleen.utils.timez
# Utility functions for Baleen
#
# Author:   Benjamin Bengfort <benjamin@bengfort.com>
# Created:  Mon Sep 22 10:14:57 2014 -0400
#
# Copyright (C) 2014 Bengfort.com
# For license information, see LICENSE.txt
#
# ID: utils.py [] benjamin@bengfort.com $

"""
Utility functions for Baleenc
"""

##########################################################################
## Imports
##########################################################################

import re

from dateutil.tz import tzlocal, tzutc
from datetime import date, datetime, timedelta

##########################################################################
## Format constants
##########################################################################

HUMAN_DATETIME   = "%a %b %d %H:%M:%S %Y %z"
HUMAN_DATE       = "%b %d, %Y"
HUMAN_TIME       = "%I:%M:%S %p"
JSON_DATETIME    = "%Y-%m-%dT%H:%M:%S.%fZ" # Must be UTC
ISO8601_DATETIME = "%Y-%m-%dT%H:%M:%S%z"
ISO8601_DATE     = "%Y-%m-%d"
ISO8601_TIME     = "%H:%M:%S"
COMMON_DATETIME  = "%d/%b/%Y:%H:%M:%S %z"

##########################################################################
## Module helper function
##########################################################################

def localnow():
    return datetime.now(tzlocal())

def utcnow():
    now = datetime.utcnow()
    now = now.replace(tzinfo=tzutc())
    return now

zre = re.compile(r'([\-\+]\d{4})')
def strptimez(dtstr, dtfmt):
    """
    Helper function that performs the timezone calculation to correctly
    compute the '%z' format that is not added by default in Python 2.7.
    """
    if '%z' not in dtfmt:
        return datetime.strptime(dtstr, dtfmt)

    dtfmt  = dtfmt.replace('%z', '')
    offset = int(zre.search(dtstr).group(1))
    dtstr  = zre.sub('', dtstr)
    delta  = timedelta(hours = offset/100)
    utctsp = datetime.strptime(dtstr, dtfmt) - delta
    return utctsp.replace(tzinfo=tzutc())



# tests.test_opml
# Testing for the OPML reader and ingestion function.
#
# Author:   Benjamin Bengfort <benjamin@bengfort.com>
# Created:  Fri Feb 19 08:50:19 2016 -0500
#
# Copyright (C) 2016 Bengfort.com
# For license information, see LICENSE.txt
#
# ID: test_opml.py [] benjamin@bengfort.com $

"""
Testing for the OPML reader and ingestion function.
"""

##########################################################################
## Imports
##########################################################################

import os
import unittest

try:
    from unittest import mock
except ImportError:
    import mock

from baleen.opml import OPML

##########################################################################
## Fixtures
##########################################################################

FIXTURES = os.path.join(os.path.dirname(__file__), "..", "fixtures")
FEEDLY   = os.path.join(FIXTURES, "feedly.opml")

##########################################################################
## Test Ingestion
##########################################################################

class IngestionTests(unittest.TestCase):

    @unittest.skip("not implemented yet")
    def test_ingest_integrated(self):
        """
        Test the integration of the ingest helper function.
        """
        raise NotImplementedError("Not Implemented Yet")


##########################################################################
## OPML Reader Test
##########################################################################

class OPMLTests(unittest.TestCase):

    def test_fixture(self):
        """
        Assert the required opml fixture is available.
        """
        self.assertTrue(os.path.exists(FEEDLY))
        self.assertTrue(os.path.isfile(FEEDLY))

    def test_categories(self):
        """
        Test the OPML categories listing
        """
        opml = OPML(FEEDLY)
        expected = [
            u'cooking',
            u'cinema',
            u'gaming',
            u'tech',
            u'essays',
            u'business',
            u'design',
            u'sports',
            u'books',
            u'data science',
            u'do it yourself'
        ]

        self.assertEqual(list(opml.categories()), expected)

    def test_length(self):
        """
        Test the OPML len built in
        """
        opml = OPML(FEEDLY)
        self.assertEqual(len(opml), 102)

    def test_counts(self):
        """
        Test the OPML category counter and item iterator
        """
        opml = OPML(FEEDLY)
        expected = {
            'cooking': 9,
            'cinema': 10,
            'gaming': 8,
            'tech': 14,
            'essays': 2,
            'business': 8,
            'design': 15,
            'sports': 7,
            'books': 6,
            'data science': 15,
            'do it yourself': 8,
        }
        counts = opml.counts()

        for key, val in expected.items():
            self.assertIn(key, counts)
            self.assertEqual(counts[key], val)

    def test_item_iterator_detail(self):
        """
        Test the XML result returned from OPML iteration
        """

        opml  = OPML(FEEDLY)
        attrs = ['category', 'title', 'text', 'htmlUrl', 'xmlUrl', 'type']
        for item in opml:
            self.assertTrue(isinstance(item, dict))
            self.assertEqual(item.keys(), attrs)



# baleen.config
# Uses confire to get meaningful configurations from a yaml file
#
# Author:   Benjamin Bengfort <benjamin@bengfort.com>
# Created:  Fri Sep 19 11:14:33 2014 -0400
#
# Copyright (C) 2014 Bengfort.com
# For license information, see LICENSE.txt
#
# ID: config.py [] benjamin@bengfort.com $

"""
Uses confire to get meaningful configurations from a yaml file
"""

##########################################################################
## Imports
##########################################################################

import os
import confire

##########################################################################
## Configuration
##########################################################################

class MongoConfiguration(confire.Configuration):
    """
    Configuration for the Mongo database
    """

    host = "localhost"
    port = 27017
    name = "baleen"

class BaleenConfiguration(confire.Configuration):
    """
    Meaningful defaults and required configurations.

    debug:    the app will print or log debug statements
    database: connection information for mongo
    """

    CONF_PATHS = [
        "/etc/baleen.yaml",                      # System configuration
        os.path.expanduser("~/.baleen.yaml"),    # User specific config
        os.path.abspath("conf/baleen.yaml"),     # Local configuration
    ]

    debug      = True
    database   = MongoConfiguration()
    logfile    = 'baleen.log'                    # Location to write log
    loglevel   = 'DEBUG'                         # Log messages to record
    fetch_html = True                            # Actually fetch HTML link

## Load settings immediately for import
settings = BaleenConfiguration.load()

if __name__ == '__main__':
    print settings



#!/usr/bin/env python
# setup
# Setup script for installing baleen
#
# Author:   Benjamin Bengfort <benjamin@bengfort.com>
# Created:  Fri Sep 19 10:59:24 2014 -0400
#
# Copyright (C) 2014 Bengfort.com
# For license information, see LICENSE.txt and NOTICE.md
#
# ID: setup.py [] benjamin@bengfort.com $

"""
Setup script for installing baleen.
See http://bbengfort.github.io/programmer/2016/01/20/packaging-with-pypi.html
"""

##########################################################################
## Imports
##########################################################################

import os
import re
import codecs

from setuptools import setup
from setuptools import find_packages

##########################################################################
## Package Information
##########################################################################

## Basic information
NAME         = "baleen"
DESCRIPTION  = "An automated ingestion service for blogs to construct a corpus for NLP research."
AUTHOR       = "Benjamin Bengfort"
EMAIL        = "benjamin@bengfort.com"
LICENSE      = "MIT"
REPOSITORY   = "https://github.com/bbengfort/baleen"
PACKAGE      = "baleen"

## Define the keywords
KEYWORDS     = ('nlp', 'baleen', 'ingestion', 'blogs', 'rss')

## Define the classifiers
## See https://pypi.python.org/pypi?%3Aaction=list_classifiers
CLASSIFIERS  = (
    'Development Status :: 4 - Beta',
    'Environment :: Console',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Natural Language :: English',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2.7',
    'Topic :: Software Development',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: Utilities',
)

## Important Paths
PROJECT      = os.path.abspath(os.path.dirname(__file__))
REQUIRE_PATH = "requirements.txt"
VERSION_PATH = os.path.join(PACKAGE, "version.py")
PKG_DESCRIBE = "DESCRIPTION.txt"

## Directories to ignore in find_packages
EXCLUDES     = (
    "tests", "bin", "docs", "fixtures", "register", "notebooks",
)

##########################################################################
## Helper Functions
##########################################################################

def read(*parts):
    """
    Assume UTF-8 encoding and return the contents of the file located at the
    absolute path from the REPOSITORY joined with *parts.
    """
    with codecs.open(os.path.join(PROJECT, *parts), 'rb', 'utf-8') as f:
        return f.read()


def get_version(path=VERSION_PATH):
    """
    Reads the __init__.py defined in the VERSION_PATH to find the get_version
    function, and executes it to ensure that it is loaded correctly.
    """
    namespace = {}
    exec(read(path), namespace)
    return namespace['get_version']()


def get_requires(path=REQUIRE_PATH):
    """
    Yields a generator of requirements as defined by the REQUIRE_PATH which
    should point to a requirements.txt output by `pip freeze`.
    """
    for line in read(path).splitlines():
        line = line.strip()
        if line and not line.startswith('#'):
            yield line

##########################################################################
## Define the configuration
##########################################################################

config = {
    "name": NAME,
    "version": get_version(),
    "description": DESCRIPTION,
    "long_description": read(PKG_DESCRIBE),
    "license": LICENSE,
    "author": AUTHOR,
    "author_email": EMAIL,
    "maintainer": AUTHOR,
    "maintainer_email": EMAIL,
    "url": REPOSITORY,
    "download_url": "{}/tarball/v{}".format(REPOSITORY, get_version()),
    "packages": find_packages(where=PROJECT, exclude=EXCLUDES),
    "install_requires": list(get_requires()),
    "classifiers": CLASSIFIERS,
    "keywords": KEYWORDS,
    "zip_safe": False,
    "scripts": ['bin/baleen'],
}

##########################################################################
## Run setup script
##########################################################################

if __name__ == '__main__':
    setup(**config)



# baleen.feed
# Handles the ingestion of documents from an RSS feed
#
# Author:   Benjamin Bengfort <benjamin@bengfort.com>
# Created:  Sun Sep 21 09:58:44 2014 -0400
#
# Copyright (C) 2014 Bengfort.com
# For license information, see LICENSE.txt
#
# ID: feed.py [] benjamin@bengfort.com $

"""
Handles the ingestion of documents from a list of RSS feeds.

This module provides two mechanisms for downloading a list of RSS feeds-
one that accepts as input an OPML file and then downloads all of the RSS
directly to disk. The other that uses data that is stored in Mongo to
fetch the feeds and then saves the documents to a Mongo collection.
"""

##########################################################################
## Imports
##########################################################################

import requests
import feedparser
import baleen.models as db

from copy import copy
from baleen.opml import OPML
from collections import Counter
from baleen.config import settings
from baleen.utils.timez import localnow
from baleen.utils.logger import IngestLogger
from dateutil import parser as dtparser

##########################################################################
## FeedIngestor Base
##########################################################################

class FeedIngestor(object):
    """
    The base class for any FeedIngestor.
    """

    feed_urls = None

    def get_feed_urls(self):
        """
        Should yield a list of feeds that should go out and be fetched.
        """
        if self.feed_urls:
            for url in self.feed_urls:
                yield url
        else:
            raise NotImplementedError((
                "Subclasses must either provide a list of "
                "feed_urls or override get_feed_urls."
            ))

    def feeds(self):
        """
        Returns the feedparser feed for each url in the ingestor.
        """
        for url in self.get_feed_urls():
            yield feedparser.parse(url)

    def posts(self):
        """
        An iterable that uses the list of feeds to fetch posts.
        """
        for feed in self.feeds():
            for entry in feed.entries:
                yield self.wrangle(entry)

    def wrangle(self, entry):
        """
        Handles each entry from the feed and converts to standard data.
        Metholodolgy of wrangling is as follows:

            - all fields are kept in the entry except `published` and
              `published_parsed` since these many not contain TZ data -
              instead these two fields are replaced by `pubdate`. If there
              is no publication date, `pubdate` is set to None.

            - the tags field, if it exists, is converted to a list of
              strings. Although this may cause some data loss; it will
              make tagging of all posts simpler for the application.

            - link will be renamed url

            - content will be populated with summary, if content does not
              exist in the feed. Supposedly feedparser was already doing
              this, but it appears to not be regular.

            - title, url, content, and tags will all be encoded UTF-8.

        All of the rest of the fields will be retained after these changes
        for subclasses to do with as they please. See the models.Post for
        more information on the standard data structure.
        """
        post = entry.copy()

        ## Handle the pubdate and published strings
        if 'published_parsed' in post: del post['published_parsed']
        post['pubdate'] = dtparser.parse(post.pop('published')) if 'published' in post else None

        ## Handle the updated parsed string
        if 'updated' in post: del post['updated']
        if 'created' in post: del post['created']
        if 'updated_parsed' in post: del post['updated_parsed']
        if 'created_parsed' in post: del post['created_parsed']
        if 'expired_parsed' in post: del post['created_parsed']

        ## Handle the tags in the entry
        post['tags'] = [tag['term'] for tag in entry.tags] if 'tags' in entry else []

        ## Rename the link field to url
        post['url']  = entry.link or post.get('href', None) or entry.id
        if 'link' in post: del post['link']

        ## Handle the content
        if 'content' not in post:
            post['content'] = entry.get('summary')
        else:
            selected = None
            for idx, item in enumerate(entry['content']):
                if idx == 0:
                    # Take the first item
                    selected = item
                elif item['type'] == 'text/html':
                    # Unless we find another item that is html
                    selected = item
            post['language'] = selected.get('language')
            post['mimetype'] = selected.get('type')
            post['content']  = selected.get('value')

        ## Fetch the content if requested.
        if settings.fetch_html:
            page = self.fetch(post.get('url'))
            if page:
                post['content'] = page

        return post

    def fetch(self, url):
        """
        Fetches the given url and returns the content, capturing errors.
        """
        try:
            response = requests.get(url)
            if response.status_code == 200:
                return response.text
            else:
                self.logger.error(
                    "Could not fetch '{}': {} {}".format(
                        url, response.status_code, response.reason
                    )
                )
        except Exception as e:
            self.logger.error(
                "Could not fetch '{}': {}".format(
                    url, str(e)
                )
            )

    def fields(self):
        """
        Returns a count of all the available fields in every entry.
        """
        meta = Counter()
        flds = Counter()

        for feed in self.feeds():
            feed = feedparser.parse(feed['link'])

            meta[feed.version] += 1
            meta['feeds'] += 1

            for entry in feed.entries:
                meta['entries'] += 1
                for key in entry.keys():
                    flds[key] += 1

        meta = dict(meta)
        meta['fields'] = dict(flds)
        return meta

    def ingest(self):
        """
        Intended as the entry point for ingestion
        """
        raise NotImplementedError("Subclasses must implement")

    def __len__(self):
        return sum(1 for feed in self.feeds())

##########################################################################
## MongoFeedIngestor
##########################################################################

class MongoFeedIngestor(FeedIngestor):
    """
    Uses the Mongo feed collection to go out and ingest posts.
    """

    def __init__(self, **kwargs):
        self.logger = kwargs.get('logger', IngestLogger())
        self.status = "PENDING"
        self.current_feed = None
        self.counts = None

    def get_feed_urls(self):
        """
        Accesses the feed collection for the list of feeds.

        NOTE: You must connect to the MongoDB before calling this method!
        """
        for feed in db.Feed.objects(active=True).only('link'):
            yield feed.link

    def update_feed(self, feed, result):
        """
        Updates a feed with information from the feedparser result.
        """
        feed.fetched = localnow()
        if 'etag' in result:     feed.etag     = result.etag
        if 'modified' in result: feed.modified = result.modified
        if 'version' in result:  feed.version  = result.version
        if 'href' in result:     feed.link     = result.href

        for key, value in result.feed.items():
            if key in ('updated', 'updated_parsed', 'id',
                       'published', 'published_parsed', 'category'):
                # Ignore these generated or protected fields
                continue

            elif key == 'link':
                feed.urls['htmlurl'] = value

            elif key == 'links':
                for idx, link in enumerate(value):
                    if 'rel' in link:
                        feed.urls[link['rel'] + str(idx)] = link['href']
                    else:
                        feed.urls["link%i" % idx] = link['href']
            else:
                setattr(feed, key, value)

        return feed

    def feeds(self, save=True):
        """
        Returns the feedparser feed for each url in the ingestor.
        """
        for url in self.get_feed_urls():
            try:
                feed   = db.Feed.objects.get(link=url)
                result = feedparser.parse(url, etag=feed.etag, modified=feed.modified)
                if not result.entries:
                    # Nothing returned, continue
                    self.logger.warning("No entries available for feed %s (%s)" %
                                        (feed.title, feed.id))
                    continue

                # Update feed properties
                feed = self.update_feed(feed, result)
                self.current_feed = feed

                if save: feed.save()
                self.counts['feeds'] += 1

                yield result
            except Exception as e:
                self.logger.error("Feed Error for feed %s (%s): %s" % (feed.title, feed.id, str(e)))
                self.counts['errors'] += 1
                continue

    def posts(self, save=True):
        """
        For every post that is fetched by super, yields the models.Post
        """
        for idx, post in enumerate(super(MongoFeedIngestor, self).posts()):
            try:
                post = db.Post(feed=self.current_feed, **post)

                if save: post.save()
                self.counts['entries'] += 1

                yield post
            except Exception as e:
                self.logger.error("Post Error for feed %s (%s) on entry %i: %s"
                    % (self.current_feed.title, self.current_feed.id, idx, str(e)))
                self.counts['errors'] += 1
                continue

    def wrangle(self, entry):
        """
        Performs same work as super, but also removes the id field so that
        a Mongo generated ObjectID can be stored as the id.
        """
        entry = super(MongoFeedIngestor, self).wrangle(entry)
        if 'id' in entry: del entry['id']
        return entry

    def ingest(self, verbose=True):
        """
        Entry point to ingestion
        """
        self.logger.info("Starting Ingest of Feeds from Mongo to Posts in Mongo")
        self.status = "INGESTING"
        self.counts = Counter()

        for post in self.posts():
            if verbose:
                print post

        self.status  = "FINISHED"
        msg = "Finished Ingest: %(feeds)i feeds, %(entries)i entries, %(errors)i errors"
        msg = msg % self.counts
        self.logger.info(msg)
        if verbose: print msg


    def __len__(self):
        return db.Feed.objects.count()

##########################################################################
## OPMLFeedIngestor
##########################################################################

class OPMLFeedIngestor(FeedIngestor):
    """
    Uses an OPML file to go out and ingest posts
    """

    def __init__(self, path):
        self.opml = OPML(path)

    def get_feed_urls(self):
        for feed in self.opml:
            yield feed['xmlUrl']

    def __len__(self):
        return len(self.opml)

if __name__ == '__main__':
    db.connect()
    ingestor = MongoFeedIngestor()
    # ingestor = OPMLFeedIngestor('fixtures/feedly.opml')
    ingestor.ingest()



# tests.test_export
# Test the export module - to generate a corpus for machine learning.
#
# Author:   Benjamin Bengfort <benjamin@bengfort.com>
# Created:  Sun Feb 21 15:49:18 2016 -0500
#
# Copyright (C) 2016 Bengfort.com
# For license information, see LICENSE.txt
#
# ID: test_export.py [] benjamin@bengfort.com $

"""
Test the export module - to generate a corpus for machine learning.
"""

##########################################################################
## Imports
##########################################################################

import unittest

try:
    from unittest import mock
except ImportError:
    import mock

from baleen.export import *



# tests.test_feed
# Test the feed module - the main entry point to Baleen
#
# Author:   Benjamin Bengfort <benjamin@bengfort.com>
# Created:  Sun Feb 21 15:49:18 2016 -0500
#
# Copyright (C) 2016 Bengfort.com
# For license information, see LICENSE.txt
#
# ID: test_feed.py [] benjamin@bengfort.com $

"""
Test the feed module - the main entry point to Baleen
"""

##########################################################################
## Imports
##########################################################################

import os
import unittest

try:
    from unittest import mock
except ImportError:
    import mock

from baleen.feed import *
from urlparse import urlparse

##########################################################################
## Feed Ingestor Tests
##########################################################################

class FeedIngestorTests(unittest.TestCase):
    """
    Test the feed ingestor base class.
    """

    def test_interface(self):
        """
        Assert that the feed ingestor is an interface.
        """
        ingestor = FeedIngestor()
        with self.assertRaises(NotImplementedError):
            feeds = list(ingestor.get_feed_urls())

        with self.assertRaises(NotImplementedError):
            feeds = ingestor.ingest()

        ingestor.feed_urls = ['a', 'b', 'c']
        self.assertEqual(list(ingestor.get_feed_urls()), ingestor.feed_urls)


##########################################################################
## Fixtures
##########################################################################

FIXTURES = os.path.join(os.path.dirname(__file__), "..", "fixtures")
FEEDLY   = os.path.join(FIXTURES, "feedly.opml")

##########################################################################
## OPML Feed Ingestor Tests
##########################################################################

class OPMLFeedIngestorTests(unittest.TestCase):
    """
    Test the OPML feed ingestor.
    """

    def test_get_feed_urls(self):
        """
        Test the feed url generator on an OPML feed ingestor.
        """

        ingestor = OPMLFeedIngestor(FEEDLY)
        feeds = 0

        for feed in ingestor.get_feed_urls():
            feeds += 1
            url = urlparse(feed)
            self.assertNotEqual(url.scheme, '')
            self.assertNotEqual(url.netloc, '')
            self.assertNotEqual(url.path, '')

        self.assertEqual(feeds, 102)

    def test_len(self):
        """
        Test the length method on the OPML feed ingestor.
        """

        ingestor = OPMLFeedIngestor(FEEDLY)
        self.assertEqual(len(ingestor), 102)
